pub struct TreePath<const N: usize>(pub Seq<int>);Expand description
A sequence of child indices describing a path from a starting node to a target node.
Each element selects one child at the corresponding tree level.
N is the maximum number of children of each node, so every index is less than N.
Tuple Fields
0: Seq<int>Implementations
Sourceimpl<const N: usize> TreePath<N>
impl<const N: usize> TreePath<N>
Sourcepub open spec fn spec_index(self, i: int) -> int
pub open spec fn spec_index(self, i: int) -> int
{ self.index(i) }Sourcepub open spec fn inv(self) -> bool
pub open spec fn inv(self) -> bool
{
&&& N > 0
&&& forall |i: int| 0 <= i < self.len() ==> Self::elem_inv(#[trigger] self[i])
}Sourcepub broadcast proof fn lemma_index_satisfies_elem_inv(self, i: int)
pub broadcast proof fn lemma_index_satisfies_elem_inv(self, i: int)
requires
self.inv(),0 <= i < self.len(),ensuresSelf::elem_inv(self[i]),Sourcepub broadcast proof fn lemma_empty_satisfies_inv(self)
pub broadcast proof fn lemma_empty_satisfies_inv(self)
requires
N > 0,#[trigger] self.is_empty(),ensuresself.inv(),Sourcepub open spec fn pop_head(self) -> (int, TreePath<N>)
pub open spec fn pop_head(self) -> (int, TreePath<N>)
recommends
!self.is_empty(),{ (self[0], TreePath(self.0.drop_first())) }Sourcepub broadcast proof fn lemma_pop_head_preserves_inv(self)
pub broadcast proof fn lemma_pop_head_preserves_inv(self)
requires
self.inv(),!self.is_empty(),ensuresself.pop_head().1.inv(),Sourcepub broadcast proof fn lemma_pop_head_index(self)
pub broadcast proof fn lemma_pop_head_index(self)
requires
self.inv(),!self.is_empty(),ensuresself.pop_head().0 == self[0],Sourcepub open spec fn pop_tail(self) -> (int, TreePath<N>)
pub open spec fn pop_tail(self) -> (int, TreePath<N>)
recommends
!self.is_empty(),{ (self[self.len() - 1], TreePath(self.0.drop_last())) }Sourcepub broadcast proof fn lemma_pop_tail_preserves_inv(self)
pub broadcast proof fn lemma_pop_tail_preserves_inv(self)
requires
self.inv(),!self.is_empty(),ensuresself.pop_tail().1.inv(),Sourcepub broadcast proof fn lemma_pop_tail_index(self)
pub broadcast proof fn lemma_pop_tail_index(self)
requires
self.inv(),!self.is_empty(),ensures(#[trigger] self.pop_tail()).0 == self[self.len() - 1],Sourcepub open spec fn push_head(self, hd: int) -> TreePath<N>
pub open spec fn push_head(self, hd: int) -> TreePath<N>
recommends
0 <= hd < N,{ TreePath(seq![hd].add(self.0)) }Sourcepub broadcast proof fn lemma_push_head_index(self, hd: int)
pub broadcast proof fn lemma_push_head_index(self, hd: int)
requires
self.inv(),0 <= hd < N,ensuresself.push_head(hd)[0] == hd,forall |i: int| 0 <= i < self.len() ==> #[trigger] self.push_head(hd)[i + 1] == self[i],Sourcepub broadcast proof fn lemma_push_head_preserves_inv(self, hd: int)
pub broadcast proof fn lemma_push_head_preserves_inv(self, hd: int)
requires
self.inv(),0 <= hd < N,ensuresself.push_head(hd).inv(),Sourcepub open spec fn push_tail(self, val: int) -> TreePath<N>
pub open spec fn push_tail(self, val: int) -> TreePath<N>
recommends
0 <= val < N,{ TreePath(self.0.push(val)) }Sourcepub broadcast proof fn lemma_push_tail_len(self, val: int)
pub broadcast proof fn lemma_push_tail_len(self, val: int)
requires
self.inv(),0 <= val < N,ensuresself.push_tail(val).len() == self.len() + 1,Sourcepub broadcast proof fn lemma_push_tail_index(self, val: int)
pub broadcast proof fn lemma_push_tail_index(self, val: int)
requires
self.inv(),0 <= val < N,ensuresself.push_tail(val)[self.len() as int] == val,forall |i: int| 0 <= i < self.len() ==> #[trigger] self.push_tail(val)[i] == self[i],Sourcepub broadcast proof fn lemma_push_tail_preserves_inv(self, val: int)
pub broadcast proof fn lemma_push_tail_preserves_inv(self, val: int)
requires
self.inv(),0 <= val < N,ensuresself.push_tail(val).inv(),Sourcepub broadcast proof fn lemma_new_preserves_inv(path: Seq<int>)
pub broadcast proof fn lemma_new_preserves_inv(path: Seq<int>)
requires
N > 0,forall |i: int| 0 <= i < path.len() ==> Self::elem_inv(#[trigger] path[i]),ensures(#[trigger] Self::new(path)).inv(),Auto Trait Implementations
impl<const N: usize> Freeze for TreePath<N>
impl<const N: usize> RefUnwindSafe for TreePath<N>
impl<const N: usize> Send for TreePath<N>
impl<const N: usize> Sync for TreePath<N>
impl<const N: usize> Unpin for TreePath<N>
impl<const N: usize> UnsafeUnpin for TreePath<N>
impl<const N: usize> UnwindSafe for TreePath<N>